/* wptListView.cpp - Dynamic list view control
 *	Copyright (C) 2000-2006 Timo Schulz
 *	Copyright (C) 2004 Andreas Jobs
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <windows.h>
#include <commctrl.h>

#include "wptCommonCtl.h"
#include "wptW32API.h"
#include "wptVersion.h"
#include "wptErrors.h"
#include "wptTypes.h"
#include "wptGPG.h"
#include "wptKeylist.h"
#include "resource.h"


int
listview_new (listview_ctrl_t *ctx)
{
    struct listview_ctrl_s *c;

    c = new struct listview_ctrl_s;
    if (!c)
	BUG (NULL);
    c->cols = 0;
    c->items = 0;
    c->ctrl = NULL;
    c->hil = NULL;
    c->ext_chkbox = 0;
    *ctx = c;
    return 0;
}


void
listview_release (listview_ctrl_t ctx)
{
    if (!ctx)
	return;

    ctx->cols = 0;
    ctx->ctrl = NULL;
    ctx->items = 0;
    if (ctx->hil)
	ImageList_Destroy (ctx->hil);
    delete ctx;
} /* listview_release */


int
listview_add_column (listview_ctrl_t ctx, listview_column_t col)
{	
    int rc = 0;
    LV_COLUMN lvc;

    memset( &lvc, 0, sizeof lvc );
    lvc.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
    lvc.pszText = col->fieldname;
    lvc.cx = col->width;
    lvc.fmt = LVCFMT_LEFT;
    lvc.iSubItem = col->pos;
    if( ListView_InsertColumn (ctx->ctrl, col->pos, &lvc) == -1)
	rc = 1;
    ctx->cols++;
    return rc;
} /* listview_add_column */


int 
listview_add_item_pos (listview_ctrl_t ctx, int pos)
{
    int rc = 0;
    LV_ITEM lvi;

    memset (&lvi, 0, sizeof lvi);
    lvi.iItem = pos;
    if (ListView_InsertItem (ctx->ctrl, &lvi) != pos)
	rc = -1;
    ctx->items++;
    return rc;
}


int
listview_add_item( listview_ctrl_t ctx, const char *text )
{
    int rc = 0;
    LV_ITEM lvi;

    memset( &lvi, 0, sizeof lvi );
    lvi.mask = LVIF_TEXT;
    lvi.pszText = (char *)text;
    rc = ListView_InsertItem( ctx->ctrl, &lvi );
    if( rc == -1 )
	rc = 1;	
    ctx->items++;
    return rc;
} /* listview_add_item */


int
listview_add_item_image (listview_ctrl_t ctx, const char *text, int image)
{
    int rc = 0;
    LV_ITEM lvi;

    memset( &lvi, 0, sizeof lvi );
    lvi.mask = LVIF_TEXT | LVIF_IMAGE;
    lvi.pszText = (char *)text;
    lvi.iImage = image;
    rc = ListView_InsertItem (ctx->ctrl, &lvi);
    if (rc == -1)
	rc = 1;	
    ctx->items++;
    return rc;
}


int
listview_add_item2 (listview_ctrl_t ctx, const char *text, void *magic)
{
    int rc = 0;
    LVITEM lvi;

    memset (&lvi, 0, sizeof (lvi));
    lvi.mask = LVIF_TEXT | LVIF_PARAM;
    lvi.pszText = (char *)text;
    lvi.lParam = (LPARAM)magic;
    rc = ListView_InsertItem (ctx->ctrl, &lvi);
    if (rc == -1)
	rc = 1;	
    ctx->items++;
    return rc;
} /* listview_add_item2 */


void*
listview_get_item2 (listview_ctrl_t ctx, int pos)
{
    LVITEM lvi;

    memset (&lvi, 0, sizeof lvi);
    lvi.mask = LVIF_PARAM;
    lvi.iItem = pos;
    ListView_GetItem (ctx->ctrl, &lvi);
    return (void*)lvi.lParam;
}

int
listview_set_item2 (listview_ctrl_t ctx, int pos, void *magic)
{
    LVITEM lvi;

    memset (&lvi, 0, sizeof (lvi));
    lvi.mask = LVIF_PARAM;
    lvi.iItem = pos;
    lvi.lParam = (LPARAM)magic;
    if (ListView_SetItem (ctx->ctrl, &lvi) == -1)
	return 1;
    return 0;
}


void
listview_add_sub_item( listview_ctrl_t ctx, int pos, int col, const char *text )
{	
    ListView_SetItemText( ctx->ctrl, pos, col, (char *)text );
} /* listview_add_sub_item */


int
listview_count_items( listview_ctrl_t ctx, int curr_sel )
{	
    int n;

    n = curr_sel? ListView_GetSelectedCount( ctx->ctrl ) :    
     	          ListView_GetItemCount( ctx->ctrl );
    return n;
} /* listview_count_items */


int
listview_del_column (listview_ctrl_t ctx, int pos)
{
    ctx->cols--;
    return ListView_DeleteColumn (ctx->ctrl, pos)? 0 : 1;
}


/* Delete a single item from @ctx at position @pos. */
int
listview_del_item (listview_ctrl_t ctx, int pos)
{
    int rc = 0;

    if (ListView_DeleteItem (ctx->ctrl, pos) == -1)
	rc = 1;
    return rc;
}


/* Delete all selected items in @ctx. */
int
listview_del_sel_items (listview_ctrl_t ctx)
{	
    int i, n;

    n = listview_count_items (ctx, 0);
    for (i = n;  i > -1; i--) {
	if (listview_get_item_state (ctx, i))
	    listview_del_item (ctx, i);
    }
    return 0;
}


int
listview_del_all_items (listview_ctrl_t ctx)
{	
    int rc = 0;

    if (ListView_DeleteAllItems (ctx->ctrl) == FALSE)
	rc = 1;
    return rc;
}


/* Return if the given element is selected or if the ext
   style is used, if the checkbox is activated. */
int
listview_get_item_state (listview_ctrl_t ctx, int pos)
{
    int ret;
    if (ctx->ext_chkbox)
	ret = ListView_GetCheckState (ctx->ctrl, pos);
    else
	ret = ListView_GetItemState (ctx->ctrl, pos, LVIS_SELECTED);
    return ret;
}


int
listview_sort_items( listview_ctrl_t ctx, int sortby, listview_cmp sort_cb )
{	
    HWND hheader;
    HDITEM hdi;
    int idx;

    ListView_SortItems (ctx->ctrl, sort_cb, sortby);
    hheader = ListView_GetHeader (ctx->ctrl);
    if (!hheader)
        return 0;

    /* Remove image from all header fields */
    memset (&hdi, 0, sizeof(hdi));

    for (int i=0; i < 7; i++) {
        hdi.mask = HDI_FORMAT;
        Header_GetItem (hheader, i, &hdi);
        hdi.fmt &= ~HDF_IMAGE;
        Header_SetItem (hheader, i, &hdi);
    }

    switch (sortby & ~KEYLIST_SORT_DESC) {
    case KEY_SORT_USERID:    idx = 0; break;
    case KEY_SORT_KEYID:     idx = 1; break;
    case KEY_SORT_IS_SECRET: idx = 2; break;
    case KEY_SORT_LEN:       idx = 3; break;
    case KEY_SORT_VALIDITY:  idx = 5; break;
    case KEY_SORT_OTRUST:    idx = 6; break;
    case KEY_SORT_CREATED:   idx = 7; break;
    case KEY_SORT_ALGO:      idx = 8; break;
    default:                 idx = 0; break;
    }

    /* Set image to currently sorted field */
    memset (&hdi, 0, sizeof(hdi));
    hdi.mask = HDI_IMAGE | HDI_FORMAT;
    Header_GetItem (hheader, idx, &hdi);
    hdi.fmt |= HDF_IMAGE | HDF_BITMAP_ON_RIGHT;
    hdi.iImage = imagelist_getindex((sortby & KEYLIST_SORT_DESC) ? 
				    IMI_SORT_DOWNARROW : IMI_SORT_UPARROW);
    Header_SetItem (hheader, idx, &hdi);
    return 0;
} /* listview_sort_items */


int
listview_get_curr_pos( listview_ctrl_t ctx )
{	
    return ListView_GetNextItem( ctx->ctrl, -1, LVNI_SELECTED );
} /* listview_get_curr_pos */


int
listview_get_item_text (listview_ctrl_t ctx, int pos, int col, char *text,
		        int maxbytes)
{
    if (pos == -1) {
	pos = listview_get_curr_pos (ctx);
	if (pos == -1)
	    return -1;
    }
    ListView_GetItemText (ctx->ctrl, pos, col, text, maxbytes);
    return 0;
} /* listview_get_item_text */


/* Use extended style with checkboxes for each item. */
void
listview_set_chkbox_style (listview_ctrl_t ctx)
{
    ListView_SetExtendedListViewStyle (ctx->ctrl, LVS_EX_CHECKBOXES);
    ctx->ext_chkbox = 1;
}


/* Use extended style to select the entire row. */
void
listview_set_ext_style (listview_ctrl_t ctx)
{	
    ListView_SetExtendedListViewStyle (ctx->ctrl, LVS_EX_FULLROWSELECT);
}


int
listview_set_column_order( listview_ctrl_t ctx, int *array )
{	
    return ListView_SetColumnOrderArray( ctx->ctrl, ctx->cols, array );
} /* listview_set_column_order */


void
listview_select_all (listview_ctrl_t ctx)
{	
    ListView_SetItemState( ctx->ctrl, -1, LVIS_SELECTED, LVIS_SELECTED );
} /* listview_select_all */


void
listview_deselect_all (listview_ctrl_t ctx)
{
    ListView_SetItemState (ctx->ctrl, -1, ~LVNI_SELECTED, LVNI_SELECTED);
}


void
listview_select_one (listview_ctrl_t ctx, int pos)
{
    ListView_SetItemState (ctx->ctrl, pos, LVIS_SELECTED|LVIS_FOCUSED, LVIS_FOCUSED|LVIS_SELECTED);
}


void
listview_scroll (listview_ctrl_t ctx, int oldpos, int newpos)
{    
    const int size=14;
    
    if (oldpos == -1)
	oldpos = 0;
    //log_box ("debug", 0, "oldpos=%d newpos=%d diff=%d", oldpos, newpos, newpos-oldpos);
    ListView_Scroll (ctx->ctrl,	0, (newpos-oldpos)*size);
}


int
listview_find (listview_ctrl_t ctx, const char * str)
{
    LVFINDINFO inf;
    int pos;

    memset (&inf, 0, sizeof (inf));
    inf.flags = LVFI_STRING|LVFI_PARTIAL;
    inf.psz = str;
    pos = ListView_FindItem (ctx->ctrl, -1, &inf);
    return pos;
}


void
listview_setview (listview_ctrl_t ctx, DWORD view)
{
    DWORD style = GetWindowLong (ctx->ctrl, GWL_STYLE);
    if ((style & LVS_TYPEMASK) != view)
	SetWindowLong (ctx->ctrl, GWL_STYLE, (style & ~LVS_TYPEMASK) | view);
}


void
listview_set_image_list (listview_ctrl_t ctx, HICON *ico, DWORD nicons)
{
    HIMAGELIST hil;
    DWORD i;

    hil = ImageList_Create (16, 16, ILC_COLOR8|ILC_MASK, nicons, 1);
    ImageList_SetBkColor (hil, CLR_NONE);
    for (i=0; i < nicons; i++)
	ImageList_AddIcon (hil, ico[i]);    
    ListView_SetImageList (ctx->ctrl, hil, LVSIL_SMALL);    
}
    
